//############################################################################
//
// EZFB ~ Linux Frame Buffer API ~
//
// by James Lehman
// james@akrobiz.com
//
// began: Feb. 2001
//
// EZFB is intended to give C programmers an easy-to-use library of functions
// for drawing points, lines and strings of characters into the Linux frame
// buffer. It also provides a means of displaying bitmap images on any portion
// of the screen and capturing the contents of any portion of the screen to
// a standard bitmap graphics file. All of this is embodied in a set of
// functions that are polymorphic with regard to the characteristics of the
// frame buffer and the bitmap files used. In other words, it makes no
// difference what the screen height or width is or what the color depth is,
// all of the function calls are the same and will have appropriate and
// predictable effects.
//
// This software is distributed in the hope that it will be useful, but
// without any warranty; without even the implied warranty of merchantability
// or fitness for a particular purpose. See the GNU General Public License
// for more details.
//
//############################################################################

#include "ezfb.h"

//############################################################################
int try_bpp       (int bpp_wanted);
int draw_stuff    (int bpp);
int draw_stuff_32 ();
int draw_stuff_24 ();
int draw_stuff_16 ();
int draw_stuff_15 ();
int draw_stuff_8  ();
int draw_stuff_4  ();
int draw_stuff_2  ();
int draw_stuff_1  ();

void display_all_bmp_types();

//############################################################################
struct frame_buffer_device  fb       = {0};
struct rectangle            bmp_area = {0};
struct rectangle            fb_area  = {0};

//############################################################################
int main(int argc, char** argv)
{
    int we_have_32 = 0;
    int we_have_24 = 0;
    int we_have_16 = 0;
    int we_have_15 = 0;
    int we_have_8  = 0;
    int we_have_4  = 0;

EZFB_DEBUGGING_FUNCTION_CALL_0

    printf("Program begins:\n");

    if(0 == init_fb(&fb))
        exit(1);

    clear_fb(&fb);

    if(argc == 1)
    {
        we_have_32 = try_bpp(32);
        sleep(2);

        we_have_24 = try_bpp(24);
        sleep(2);

        we_have_16 = try_bpp(16);
        sleep(2);

        we_have_15 = try_bpp(15);
        sleep(2); 
        we_have_8 = try_bpp(8);
        sleep(2);

        we_have_4 = try_bpp(4);
        sleep(2);
         if(we_have_32)
        {
            set_fb_bpp(&fb, 32);
            display_all_bmp_types();
        }

        if(we_have_24)
        {
            set_fb_bpp(&fb, 24);
            display_all_bmp_types();
        }

        if(we_have_16)
        {
            set_fb_bpp(&fb, 16);
            display_all_bmp_types();
        }

        if(we_have_15)
        {
            set_fb_bpp(&fb, 15);
            display_all_bmp_types();
        }

        if(we_have_8)
        {
            set_fb_bpp(&fb, 8);
            display_all_bmp_types();
        }

        if(we_have_4)
        {
            set_fb_bpp(&fb, 4);
            display_all_bmp_types();
        }

        if(we_have_8)
        {
            int i;
            set_fb_bpp(&fb, 8);
            clear_fb(&fb);
            draw_stuff_8();
            set_fb_cmap_grays(&fb);
            sleep(2);
            set_fb_cmap_reds(&fb);
            sleep(2);
            set_fb_cmap_greens(&fb);
            sleep(2);
            set_fb_cmap_blues(&fb);
            sleep(2);
            set_fb_cmap_332(&fb);
            sleep(2);
            set_fb_cmap_random(&fb);
            sleep(2);
            set_fb_cmap_smooth(&fb);
            for(i = 0; i < 256; i++)
            {
                cycle_fb_cmap(&fb, 0, 255, 1);
                usleep(10000);
            }
            set_fb_cmap_smooth(&fb);
            for(i = 0; i < 256; i++)
            {
                cycle_fb_cmap(&fb, 0, 255, 1);
                usleep(10000);
            }
            set_fb_cmap_smooth(&fb);
            for(i = 0; i < 256; i++)
            {
                cycle_fb_cmap(&fb, 0, 255, 1);
                usleep(10000);
            }

            clear_fb_cmap(&fb);

            fb_area.x1 =  10;
            fb_area.y1 =  10;
            fb_area.x2 =  -1;
            fb_area.y2 =  -1;
            bmp_open_into_fb_at(&fb, "./bmp/leon.bmp", &fb_area);

            fb_area.x1 =  10;
            fb_area.y1 = fb.max_y - 160;
            fb_area.x2 =  -1;
            fb_area.y2 =  -1;
            bmp_open_into_fb_at(&fb, "./bmp/leon.bmp", &fb_area);

            fb_area.x1 = fb.max_x - 210;
            fb_area.y1 =  10;
            fb_area.x2 =  -1;
            fb_area.y2 =  -1;
            bmp_open_into_fb_at(&fb, "./bmp/leon.bmp", &fb_area);

            fb_area.x1 = fb.max_x - 210;
            fb_area.y1 = fb.max_y - 160;
            fb_area.x2 =  -1;
            fb_area.y2 =  -1;
            bmp_open_into_fb_at(&fb, "./bmp/leon.bmp", &fb_area); 

            fb_area.x1 = fb.max_x / 2 - 100;
            fb_area.y1 = fb.max_y / 2 -  75;
            fb_area.x2 =  -1;
            fb_area.y2 =  -1;
            bmp_open_into_fb_at(&fb, "./bmp/leon.bmp", &fb_area);
        }
    }
    else
        try_bpp(atoi(argv[1]));

    put_string(&fb, "     EZFB     ", fb.max_x / 2 - 112,            60, fb.colors.black, fb.colors.white, 0, 2);
    put_string(&fb, " HIT ANY KEY! ", fb.max_x / 2 - 112, fb.max_y - 60, fb.colors.black, fb.colors.white, 0, 2);
    wait_4_key();
    release_fb(&fb);

EZFB_DEBUGGING_FUNCTION_RETURN(!ret)
}

//############################################################################
int try_bpp(int bpp_wanted)
{
EZFB_DEBUGGING_FUNCTION_CALL_1
    if((ret = set_fb_bpp(&fb, bpp_wanted)))
    {
        char file_name[42];
        ret &= clear_fb(&fb);
        ret &= draw_stuff(bpp_wanted);
        ret &= fb_printf(&fb, 12,  fb.max_y - 25,  fb.colors.black, fb.colors.white, 0, 1, "./bmp/%dbit.bmp", fb.Var.bits_per_pixel);
        printf("\n### %d x %d @ %d bpp ###\n", fb.Var.xres, fb.Var.yres, fb.Var.bits_per_pixel);
        //sprintf(file_name, "./bmp/%dx%d_%d.bmp", fb.Var.xres, fb.Var.yres, fb.Var.bits_per_pixel);
        sprintf(file_name, "./bmp/%dbit.bmp", fb.Var.bits_per_pixel);
        printf("\n%s = %d bytes\n", file_name, bmp_save_fb_as(&fb, file_name));
        ret &= dump_fb_fix_screeninfo(&fb);
        ret &= dump_fb_var_screeninfo(&fb);
        ret &= dump_ezfb_object      (&fb);
        ret &= dump_ezfb_colors      (&fb);
    }
    else
        fb_printf(&fb, 12,  fb.max_y - 25, fb.colors.black, fb.colors.white, 0, 3, "%2d bpp not supported", bpp_wanted);

EZFB_DEBUGGING_FUNCTION_RETURN(ret)
}

//############################################################################
int draw_stuff(int bpp)
{
EZFB_DEBUGGING_FUNCTION_CALL
    switch(bpp)
    {
        case  1: draw_stuff_1  (); break;
        case  2: draw_stuff_2  (); break;
        case  4: draw_stuff_4  (); break;
        case  8: draw_stuff_8  (); break;
        case 15: draw_stuff_15 (); break;
        case 16: draw_stuff_16 (); break;
        case 24: draw_stuff_24 (); break;
        case 32: draw_stuff_32 (); break;
        default:                   break;
    }
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}

//############################################################################
int draw_stuff_32()
{
    int x, y;
EZFB_DEBUGGING_FUNCTION_CALL
    for(y = 0; y < fb.Var.yres; y++)
        for(x = 0; x < fb.Var.xres; x++)
            put_pixel(&fb, x, y, make_rgb( &fb,
                                          ( ~(0x180*x/fb.Var.xres) ^ (0x180*y/fb.Var.yres) ) &  (0x180*y/fb.Var.yres),
                                          ( ~(0x180*x/fb.Var.xres) & (0x180*y/fb.Var.yres) ) -  (0x180*x/fb.Var.xres),
                                          ( ~(0x180*x/fb.Var.xres) | (0x180*y/fb.Var.yres) ) & ((0x180*y/fb.Var.yres) - (0x180*x/fb.Var.xres)) ));
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}

//############################################################################
int draw_stuff_24()
{
    int x;
EZFB_DEBUGGING_FUNCTION_CALL
    clear_fb(&fb);
    for(x = 0; x < 256; x++)
    {
        fb_line(&fb, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 +   0, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 +  59, make_rgb(&fb, x/2, x/2, x/2));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 +  60, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 119, make_rgb(&fb,   x,   0,   0));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 120, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 179, make_rgb(&fb,   0,   x,   0));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 180, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 239, make_rgb(&fb,   0,   0,   x));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 240, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 299, make_rgb(&fb,   x,   x,   0));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 300, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 359, make_rgb(&fb,   x,   0,   x));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 360, fb.Var.xres / 2 - 255 + x, fb.Var.yres / 2 - 210 + 419, make_rgb(&fb,   0,   x,   x));

        fb_line(&fb, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 +   0, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 +  59, make_rgb(&fb, 127-(255-x/2), 127-(255-x/2), 127-(255-x/2) ));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 +  60, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 119, make_rgb(&fb, 255, x, x));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 120, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 179, make_rgb(&fb, x, 255, x));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 180, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 239, make_rgb(&fb, x, x, 255));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 240, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 299, make_rgb(&fb, 255, 255, x));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 300, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 359, make_rgb(&fb, 255, x, 255));
        fb_line(&fb, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 360, fb.Var.xres / 2 - 255 + x + 254, fb.Var.yres / 2 - 210 + 419, make_rgb(&fb, x, 255, 255));
    }
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}

//############################################################################
int draw_stuff_16()
{
    int x, y;
EZFB_DEBUGGING_FUNCTION_CALL
    for(y = 0; y < fb.Var.yres; y++)
        for(x = 0; x < fb.Var.xres; x++)
            put_pixel(&fb, x, y, make_rgb(&fb, y - x, x * 5, y * 5));
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}

//############################################################################
int draw_stuff_15()
{
    int x, y;
EZFB_DEBUGGING_FUNCTION_CALL
    for(y = 0; y < fb.Var.yres; y++)
        for(x = 2; x < fb.Var.xres; x++)
            put_pixel(&fb, x, y, make_rgb(&fb, (y-x)/16, x, y));
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}

//############################################################################
int draw_stuff_8()
{
    int i, j, k, x, y;
EZFB_DEBUGGING_FUNCTION_CALL
    k = 0;
    for(i = 0; i < 16; i++)
        for(j = 0; j < 16; j++)
        {             for(y = 0; y < (fb.Var.yres / 24 - 1); y++)
                for(x = 0; x < (fb.Var.xres / 16 - 1); x++)
                    put_pixel(&fb, j * (fb.Var.xres / 16) + x, i * (fb.Var.yres / 24) + y, k);
            k++;
        }
    for(i = 0; i < fb.max_x; i++)
    {
        fb_line(&fb, i           , fb.Var.yres * 2 / 3, i           , fb.Var.yres * 5 / 6, i % 256);
        fb_line(&fb, fb.max_x - i, fb.Var.yres * 5 / 6, fb.max_x - i, fb.max_y           , i % 256);
    }
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}

//############################################################################
int draw_stuff_4()
{
    int i, j, k, x, y;
EZFB_DEBUGGING_FUNCTION_CALL
    k = 0;
    for(i = 0; i < 16; i++)
        for(j = 0; j < 16; j++)
        {
            for(y = 0; y < (fb.Var.yres / 24 - 1); y++)
                for(x = 0; x < (fb.Var.xres / 16 - 1); x++)
                    put_pixel(&fb, j * (fb.Var.xres / 16) + x, i * (fb.Var.yres / 24) + y, k);
            k++;
        }
    for(i = 0; i < fb.max_x; i++)
    {
        fb_line(&fb, i           , fb.Var.yres * 2 / 3, i           , fb.Var.yres * 5 / 6, i % 16);
        fb_line(&fb, fb.max_x - i, fb.Var.yres * 5 / 6, fb.max_x - i, fb.max_y           , i % 16);
    }
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}

//############################################################################
int draw_stuff_2()
{
EZFB_DEBUGGING_FUNCTION_CALL
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}

//############################################################################
int draw_stuff_1()
{
EZFB_DEBUGGING_FUNCTION_CALL
EZFB_DEBUGGING_FUNCTION_RETURN(1)
}


//############################################################################
void display_all_bmp_types()
{
    bmp_open_into_fb(&fb, "./bmp/24bit.bmp");
    fb_printf(&fb, 50, 50, fb.colors.black, fb.colors.white, 0, 1, "24 bpp bmp in %d bpp fb", fb.Var.bits_per_pixel);
    sleep(2);
    bmp_open_into_fb(&fb, "./bmp/8bit.bmp");
    fb_printf(&fb, 50, 50, fb.colors.black, fb.colors.white, 0, 1, "8 bpp bmp in %d bpp fb", fb.Var.bits_per_pixel);
    sleep(2);
    bmp_open_into_fb(&fb, "./bmp/4bit.bmp");
    fb_printf(&fb, 50, 50, fb.colors.black, fb.colors.white, 0, 1, "4 bpp bmp in %d bpp fb", fb.Var.bits_per_pixel);
    sleep(2);
}

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################







